unit Mmmain;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, MPlayer, Ledgadgt, StdCtrls, ExtCtrls, mmSystem, Toggler,
  Stereo, IniFiles, Menus;

type
  TTrackInfo = Class(tObject)
  Public
    FDeviceId : LongInt;
    Track     : LongInt;
    TrackName : String;
    TrackLen  : LongInt;
    TrackPlay : Boolean;
    constructor Create(wDeviceId, CurrTrack :LongInt);
  end;

type
  TDeviceInfo = Class(tObject)
  public
    FDeviceId   : LongInt;
    CDName      : String;
    TrackCount  : LongInt;
    TrackInfo   : array[1..36] of TTrackInfo;
    CdUniqueCode: String;
    constructor Create(wDeviceId: LongInt);
    destructor Destroy; override;
  end;

type
  TCdPlay = class(TForm)
    Panel1: TPanel;
    Label1: TLabel;
    LEDTrack: TLEDLabel;
    Label2: TLabel;
    Label3: TLabel;
    LEDElapsed: TLEDLabel;
    LEDRemain: TLEDLabel;
    Panel2: TPanel;
    MediaPlayer1: TMediaPlayer;
    Timer1: TTimer;
    StereoButton1: TStereoButton;
    Label4: TLabel;
    MainMenu1: TMainMenu;
    Options1: TMenuItem;
    AutoSave: TMenuItem;
    View: TMenuItem;
    TrackList: TMenuItem;
    About1: TMenuItem;
    Prorams: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure StereoButton1Click(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Label4Click(Sender: TObject);
    procedure AutoSaveClick(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure ProgramsClick(Sender: TObject);
    procedure MediaPlayer1Notify(Sender: TObject);
    procedure MediaPlayer1Click(Sender: TObject; Button: TMPBtnType;
      var DoDefault: Boolean);
  private
    DeviceInfo   : TDeviceInfo;
    DeviceOpened : Boolean;
    DeviceExist  : Boolean;
    procedure DeviceOpen;
    procedure DeviceClose;
    procedure GetDeviceInfo(DeviceInfo:TDeviceInfo);
    procedure SaveDeviceInfo(DeviceInfo:TDeviceInfo);
    Function FindNextTrack : Boolean;
    Function FindPrevTrack : Boolean;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  CdPlay: TCdPlay;

implementation
Uses GetTrack, CdAbout, CdProgr;
{$R *.DFM}


constructor TTrackInfo.Create(wDeviceId, CurrTrack :LongInt);
Var  StatusParm: TMCI_Status_Parms;
     FFlags    : Longint;
     i         : LongInt;
     FError    : Longint;

Begin
  inherited Create;
  FDeviceId  := wDeviceId;
  Track      := CurrTrack;
  TrackName  := '';
  TrackPlay  := True;
  {
  StatusParm.dwItem  := MCI_STATUS_POSITION;
  StatusParm.dwTrack := Track;
  FFlags             := mci_Status_Item or mci_Track;
  FError             := mciSendCommand(FDeviceId,mci_status ,FFlags,Longint(@StatusParm));
  }
  FFlags := mci_Wait or mci_Status_Item or mci_Track;
  StatusParm.dwItem  := mci_Status_Length;
  StatusParm.dwTrack := Track;
  FError             := mciSendCommand( FDeviceID, mci_Status, FFlags, Longint(@StatusParm));
  if FError <> 0 then
    raise EMCIDeviceError.Create(Format('Cannot read information on track nr. %2d',[Track]));
  TrackLen           := StatusParm.dwReturn;
End;

constructor TDeviceInfo.Create(wDeviceId: LongInt);
Var i         : Longint;
    FFlags    : Longint;
    StatusParm: TMCI_Status_Parms;
    SetParm   : TMCI_Set_Parms;
    FError    : Longint;
begin
  inherited Create;
  FDeviceId         := wDeviceId;
  CDName            := '';

  {Setta il tempo in minuti, secondi, frame}
  FFlags               := mci_Notify or mci_Set_Time_Format;
  SetParm.dwTimeFormat := Longint(tfMSF);
  FError               := mciSendCommand( FDeviceID, mci_Set, FFlags, Longint(@SetParm) );
  if FError <> 0 then
    raise EMCIDeviceError.Create('Cannot change time to MSF');

  {Conta le tracce}
  FFlags            := mci_Wait or mci_Status_Item;
  StatusParm.dwItem := mci_Status_Number_Of_Tracks;
  FError            := mciSendCommand( FDeviceId, mci_Status, FFlags, Longint(@StatusParm));
  if FError <> 0 then
    raise EMCIDeviceError.Create('Cannot count device tracks');

  TrackCount        := StatusParm.dwReturn;
  For i := 1 to TrackCount do
  Begin
    TrackInfo[i] := tTrackInfo.Create(FDeviceId,i);
    With TrackInfo[i] do
    Begin
      TrackName := Format('Track %2d',[i]);
      CdUniqueCode := CdUniqueCode + IntToStr(mci_MSF_Minute(TrackLen)) +
                                     IntToStr(mci_MSF_Second(TrackLen)) ;
    End
  End;

  {Setta il tempo in tracce, minuti, secondi, frame}
  FFlags               := mci_Notify or mci_Set_Time_Format;
  SetParm.dwTimeFormat := Longint(tfTMSF);
  FError               := mciSendCommand( FDeviceID, mci_Set, FFlags, Longint(@SetParm) );
  if FError <> 0 then
    raise EMCIDeviceError.Create('Cannot change time to TMSF');
End;

destructor TDeviceInfo.Destroy;
Var i         : Longint;
Begin
  For i := 1 to TrackCount do
  Begin
    TrackInfo[i].Free;
  End;
End;

procedure TCdPlay.GetDeviceInfo(DeviceInfo:TDeviceInfo);
Var IniFile : tIniFile;
    Tracce  : tStringList;
    I       : Integer;
begin
  Tracce := tStringList.Create;
  IniFile := tIniFile.Create('MULTMED.INI');
  IniFile.ReadSectionValues(DeviceInfo.CDUniqueCode, Tracce);
  IniFile.Free;
  If Tracce.Count = 0 then
  Begin
    DeviceInfo.CDName := 'New CD';
    DeviceExist := False;
  End
  Else
  Begin
    DeviceInfo.CDName := Tracce.Values['CDName'];
    DeviceExist := True;
    For i := 1 to DeviceInfo.TrackCount do
    Begin
      DeviceInfo.TrackInfo[i].TrackName := Tracce.Values[Format('Track_%2d',[i])];
    End;
  End;
  Tracce.Free;
  Self.Caption := 'Cd Player - ' + DeviceInfo.CDName;
end;

procedure TCdPlay.SaveDeviceInfo(DeviceInfo:TDeviceInfo);
Var IniFile : tIniFile;
    I       : Integer;
begin
  IniFile := tIniFile.Create('MULTMED.INI');
  IniFile.EraseSection(DeviceInfo.CDUniqueCode);
  IniFile.WriteString(DeviceInfo.CDUniqueCode, 'CDName' , DeviceInfo.CDName);
  For i := 1 to DeviceInfo.TrackCount do
  Begin
    with DeviceInfo do
      IniFile.WriteString(CDUniqueCode, Format('Track_%2d',[i]),TrackInfo[i].TrackName);
  End;
  IniFile.Free;
end;

procedure TCdPlay.FormCreate(Sender: TObject);
begin
  DeviceOpened := False;
  DeviceOpen;
end;

procedure TCdPlay.DeviceOpen;
begin
  Try
    MediaPlayer1.DeviceType := dtCDAudio;
    MediaPlayer1.Wait := True;
    MediaPlayer1.TimeFormat := tfTMSF;
    MediaPlayer1.Open;
    StereoButton1.IsOn := True;
  Finally
    MediaPlayer1.Wait := False;
  End;
end;

procedure TCdPlay.DeviceClose;
begin
  MediaPlayer1.Close;
  StereoButton1.IsOn := False;
end;

procedure TCdPlay.StereoButton1Click(Sender: TObject);
begin
  if StereoButton1.IsOn then DeviceOpen else DeviceClose;
end;

procedure TCdPlay.Timer1Timer(Sender: TObject);
var  sec2,min2,sec,min,tr:byte;
     TheLength : LongInt;
     TheElaps  : LongInt;
     TheRemain : LongInt;
begin
  sec := mci_TMSF_Second(Mediaplayer1.Position);
  min := mci_TMSF_Minute(Mediaplayer1.Position);
  tr  := mci_TMSF_Track(Mediaplayer1.Position);
  If Mediaplayer1.Error <> 0 then
  Begin
    If DeviceOpened then
    Begin
      If AutoSave.Checked then
      Begin
        SaveDeviceInfo(DeviceInfo);
      End;
      DeviceInfo.Free;
    End;
    DeviceOpened := False;
    LEDTrack.Caption   := 'N/A';
    LEDElapsed.Caption := ' N/A ';
    LEDRemain.Caption  := ' N/A ';
    Self.Caption       := 'Cd Player - Unknown';
    Label4.Caption     := 'Unknown';
    View.Enabled  := False;
  End
  else
  Begin
    if not DeviceOpened then
    Begin
      DeviceOpened := True;
      DeviceInfo := TDeviceInfo.Create(Mediaplayer1.DeviceID);
      GetDeviceInfo(DeviceInfo);
      View.Enabled  := True;
      MCISendCommand(Mediaplayer1.DeviceID,MCI_SEEK,MCI_SEEK_TO_START,0);
    End;
    TheElaps  := (Sec + (Min*60));
    sec2 := mci_MSF_Second(DeviceInfo.TrackInfo[tr].TrackLen);
    min2 := mci_MSF_Minute(DeviceInfo.TrackInfo[tr].TrackLen);
    TheLength := (Sec2 + (Min2*60));
    TheRemain := TheLength - TheElaps;
    min2 := TheRemain div 60;
    sec2 := TheRemain - (min2 * 60);
    Label4.Caption     := DeviceInfo.TrackInfo[tr].TrackName;
    LEDTrack.Caption   := Format('%3d',[tr]);
    LEDElapsed.Caption := Format('%2.2d:%2.2d',[Min,Sec]);
    LEDRemain.Caption  := Format('%2.2d:%2.2d',[Min2,Sec2]);
  end;
End;


procedure TCdPlay.FormDestroy(Sender: TObject);
begin
  DeviceClose;
  If DeviceOpened then
  Begin
    DeviceOpened := False;
    If AutoSave.Checked then
    Begin
      SaveDeviceInfo(DeviceInfo);
    End;
    DeviceInfo.Free;
    View.Enabled := False;
  End
end;

procedure TCdPlay.Label4Click(Sender: TObject);
Var Modal : TEditTrack;
    i     : Integer;
begin
  Try
    Modal := TEditTrack.Create(Self);
    Modal.CdName.Text := DeviceInfo.CDName;
    Modal.EleTrack.RowCount := DeviceInfo.TrackCount;
    For i := 1 to DeviceInfo.TrackCount do
    Begin
      Modal.EleTrack.Cells[0,i - 1] := DeviceInfo.TrackInfo[i].TrackName;
    End;
    if Modal.ShowModal = mrOk then
    Begin
      DeviceInfo.CDName := Modal.CdName.Text;
      For i := 1 to DeviceInfo.TrackCount do
      Begin
        DeviceInfo.TrackInfo[i].TrackName := Modal.EleTrack.Cells[0,i - 1];
      End;
    End;
  Finally
    Modal.Free
  End;
end;

procedure TCdPlay.AutoSaveClick(Sender: TObject);
begin
  AutoSave.Checked := not AutoSave.Checked;
end;

procedure TCdPlay.About1Click(Sender: TObject);
var
  AboutBox: TAboutBox;
begin
  Try
    AboutBox := TAboutBox.Create(Self);
    AboutBox.ShowModal
  Finally
    AboutBox.Free
  End;
end;

procedure TCdPlay.ProgramsClick(Sender: TObject);
Var Modal : TProgramTrack;
    i     : Integer;
    MkPrg : Boolean;
begin
  Try
    Modal := TProgramTrack.Create(Self);
    For i := 1 to DeviceInfo.TrackCount do
    Begin
      if DeviceInfo.TrackInfo[i].TrackPlay then
        Modal.DstList.Items.Add(DeviceInfo.TrackInfo[i].TrackName)
      else
        Modal.SrcList.Items.Add(DeviceInfo.TrackInfo[i].TrackName);
    End;
    MkPrg := not (Modal.SrcList.Items.Count = 0);
    Modal.SetButtons;
    if Modal.ShowModal = mrOk then
    Begin
      MkPrg := not (Modal.SrcList.Items.Count = 0);
      For i := 1 to DeviceInfo.TrackCount do
      Begin
        DeviceInfo.TrackInfo[i].TrackPlay := (Modal.SrcList.items.IndexOf(DeviceInfo.TrackInfo[i].TrackName) < 0);
      End;
    End;
  Finally
    Modal.Free
  End;
  if MkPrg then
  Begin
    MediaPlayer1.Tag := 0;
    FindNextTrack;
  End
  Else
  Begin
    MediaPlayer1.Notify := False;
    MediaPlayer1.Tag := 0;
  End;
end;

procedure TCdPlay.MediaPlayer1Notify(Sender: TObject);
Var Found : Boolean;
begin
  If MediaPlayer1.NotifyValue = nvSuccessful then
  Begin
    Found := FindNextTrack;
    If Found then
    Begin
      MediaPlayer1.Play;
    End;
  End;
end;

procedure TCdPlay.MediaPlayer1Click(Sender: TObject; Button: TMPBtnType;
  var DoDefault: Boolean);
begin
  DoDefault := True;
  If Button = btNext then
  Begin
    If Mediaplayer1.Tag > 0 then
    Begin
      DoDefault := False;
      FindNextTrack;
      if MediaPlayer1.Mode = mpPlaying then
      begin
        MediaPlayer1.Play;
        Exit;
      end
    End;
  End;

  If Button = btPrev then
  Begin
    If Mediaplayer1.Tag > 0 then
    Begin
      DoDefault := False;
      FindPrevTrack;
      if MediaPlayer1.Mode = mpPlaying then
      begin
        MediaPlayer1.Play;
        Exit;
      end
    End;
  End;
end;

Function TCdPlay.FindNextTrack : Boolean;
Var Found : Integer;
    i     : Integer;
    SeekParm: TMCI_Seek_Parms;
    TempFlags: Longint;
Begin
  Found := 0;
  For i := MediaPlayer1.Tag + 1 to DeviceInfo.TrackCount do
  Begin
    if DeviceInfo.TrackInfo[i].TrackPlay then
    Begin
      Found := i;
      Break;
    End;
  End;
  If Found = 0 then
  Begin
    For i := 1 to DeviceInfo.TrackCount do
    Begin
      if DeviceInfo.TrackInfo[i].TrackPlay then
      Begin
        Break;
      End;
    End;
  End;

  MediaPlayer1.StartPos := mci_Make_TMSF(i, 0, 0, 0);
  If i < DeviceInfo.TrackCount then
    MediaPlayer1.EndPos := mci_Make_TMSF(i+1, 0, 0, 0);
  MediaPlayer1.Notify := True;
  MediaPlayer1.Tag := i;

  TempFlags := mci_To;
  SeekParm.dwTo := MediaPlayer1.StartPos;
  SeekParm.dwCallback := Handle;
  mciSendCommand(MediaPlayer1.DeviceId , mci_Seek, TempFlags, Longint(@SeekParm));
  Result := (Found > 0);
End;

Function TCdPlay.FindPrevTrack : Boolean;
Var Found : Integer;
    i     : Integer;
    SeekParm: TMCI_Seek_Parms;
    TempFlags: Longint;
Begin
  Found := 0;
  For i := MediaPlayer1.Tag - 1 downto 1 do
  Begin
    if DeviceInfo.TrackInfo[i].TrackPlay then
    Begin
      Found := i;
      Break;
    End;
  End;
  If Found = 0 then
  Begin
    For i := DeviceInfo.TrackCount Downto 1 do
    Begin
      if DeviceInfo.TrackInfo[i].TrackPlay then
      Begin
        Break;
      End;
    End;
  End;
  MediaPlayer1.StartPos := mci_Make_TMSF(i, 0, 0, 0);
  If i < DeviceInfo.TrackCount then
    MediaPlayer1.EndPos := mci_Make_TMSF(i+1, 0, 0, 0);
  MediaPlayer1.Notify := True;
  MediaPlayer1.Tag := i;

  TempFlags := mci_To;
  SeekParm.dwTo := MediaPlayer1.StartPos;
  SeekParm.dwCallback := Handle;
  mciSendCommand(MediaPlayer1.DeviceId , mci_Seek, TempFlags, Longint(@SeekParm));
  Result := (Found > 0);
End;

end.
